/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */
#if !defined(_INTERSECTION_VIEW_HPP)
#define _INTERSECTION_VIEW_HPP	1

#if !defined(_INTERSECTION_VIEW_FWD_HPP)
#include "intersection_view_fwd.hpp"	// validate that the forward declaration matches the actual one.
#endif

#if !defined(_FILTER_VIEW_HPP)
#include "filter_view.hpp"
#endif

#if !defined(_FUNCTORS_HPP)
#include "functors.hpp"
#endif

#if !defined(_TWO_CONTAINERS_BASE_HPP)
#include "two_containers_base.hpp"
#endif

#if !defined(_TWO_CONTAINERS_TRAITS_HPP)
#include "two_containers_traits.hpp"
#endif

BEGIN_VIEW_NAMESPACE


// typedefs to make it easier to define the intersection_view.
template <class container_a,
          class container_b, 
          class const_tag,
		  class iterator_tag,
          template <class C, class tag> class proxy_template_a,
		  template <class C, class tag> class proxy_template_b,
          class FindType_a,
          class FindType_b>
// Not really a namespace but that's all I'm using it for.
class intersection_view_namespace
{
private:
  typedef two_containers_base<container_a,container_b,const_tag,
								proxy_template_a,proxy_template_b> base_t;
  // Integrate the const_tag into the container types.
  typedef typename view_traits<container_a,const_tag>::container_type domain_a;
  typedef typename view_traits<container_b,const_tag>::container_type domain_b;
  
  typedef intersection_traits<container_a,container_b,const_tag,
					FindType_a,FindType_b> traits;
  typedef match_element<traits::search_container,
						traits::element_container,
						const_tag,
						view_ref,
						traits::FindType> predicate_fn;
  // Define a filter_view of container_a which includes any members
  // found in b.
  typedef filter_view<traits::element_container,predicate_fn,const_tag,
						iterator_tag,view_ref> base_view;
  
protected:
  // There is this one more layer so we don't have to export out
  // all of the typedefs we set up in intersection_view_namespace.
  class view : private base_t, public base_view
  {
  public:
	explicit view() {}
    view(domain_a &cont_a, domain_b &cont_b)
      :  intersection_view_namespace::base_t(cont_a,cont_b),
		  base_view(traits::getElementContainer(a(),b()),
					predicate_fn(traits::getSearchContainer(a(),b() )))
    {}
  };
  
};
template <class container_a,
          class container_b, 
          class const_tag,
		  class iterator_tag,
          class FindType_a,
          class FindType_b>
// Not really a namespace but that's all I'm using it for.
class intersection_view_namespace<container_a,container_b,const_tag,iterator_tag,view_ref,view_ref,
								FindType_a,FindType_b>
{
private:
  // Integrate the const_tag into the container types.
  typedef typename view_traits<container_a,const_tag>::container_type domain_a;
  typedef typename view_traits<container_b,const_tag>::container_type domain_b;
  
  typedef intersection_traits<container_a,container_b,const_tag,
					FindType_a,FindType_b> traits;
  typedef match_element<traits::search_container,
						traits::element_container,
						const_tag,
						view_ref,
						traits::FindType> predicate_fn;
  // Define a filter_view of container_a which includes any members
  // found in b.
  typedef filter_view<traits::element_container,predicate_fn,const_tag,
						iterator_tag,view_ref> base_view;
  
protected:
  // There is this one more layer so we don't have to export out
  // all of the typedefs we set up in intersection_view_namespace.
  class view : public base_view
  {
  public:
	explicit view() {}
    view(domain_a &cont_a, domain_b &cont_b)
      :  base_view(traits::getElementContainer(cont_a,cont_b),
					predicate_fn(traits::getSearchContainer(cont_a,cont_b )))
    {}
  };
  
};


//
// intersection_view
//
// All the elements that are in both container_a and container_b 
template <
  class container_a,
  class container_b, 
  class const_tag,
  template <class C, class tag> class proxy_template_a,
  template <class C, class tag> class proxy_template_b,
  class FindType_a,
  class FindType_b,
  class iterator_tag>
class intersection_view
	: public intersection_view_namespace<container_a,container_b, const_tag,
                 iterator_tag,proxy_template_a, proxy_template_b, FindType_a, FindType_b>::view
{
public:  
  typedef intersection_view_namespace<container_a,container_b, const_tag,
                 iterator_tag,proxy_template_a, proxy_template_b, 
				 FindType_a, FindType_b>::view inherited;
  typedef ctor_arg<container_a,const_tag,proxy_template_a>::type ctor_arg_type_a;
  typedef ctor_arg<container_b,const_tag,proxy_template_b>::type ctor_arg_type_b;
  
  explicit intersection_view() {}
  intersection_view(ctor_arg_type_a &a_cont, ctor_arg_type_b &b_cont)
    :	inherited(a_cont,b_cont) 
	{}
};

END_VIEW_NAMESPACE
#endif // _INTERSECTION_VIEW_H
