/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

#if !defined(_INTERSECTION_VIEW_FWD_HPP)
#define _INTERSECTION_VIEW_FWD_HPP

#if !defined(_VIEW_CONFIG_HPP)
#include "view_config.hpp"
#endif

#if !defined(__SGI_STL_INTERNAL_RELOPS)
#include "view_relops.hpp"
#endif

#if !defined(_VIEW_BASE_FWD_HPP)
#include "view_base_fwd.hpp"
#endif

#if !defined(_VIEW_BASE_TRAITS_HPP)
#include "view_base_traits.hpp"
#endif

#if !defined(_STL_TRAITS_HPP)
#include "stl_traits.hpp"
#endif

#if !defined(_SYS_LIBRARY_ITERATOR_)
#include <iterator>
#define _SYS_LIBRARY_ITERATOR_
#endif

// A forward declaration of every public view.
// The purpose is that if you have an include file that has a pointer to a view in
// it you don't need to include the full view.hpp file. Instead you can include just this
// file. Then the .cpp file can include the full view.hpp file and then use it.
// For each view, I have included the view and the iterator declaration if necessary.

BEGIN_VIEW_NAMESPACE

template <class container_a, class container_b,
          class const_tag,
          class FindType_a,  class FindType_b
		  >
struct intersection_traits {
  typedef view_traits<container_a,const_tag>::container_type domain_type_a;
  typedef view_traits<container_b,const_tag>::container_type domain_type_b;
  typedef domain_type_a element_container;
  typedef domain_type_b search_container;
  typedef FindType_b FindType;
  typedef typename std::iterator_traits<typename container_a::iterator>::iterator_category
	  iterator_category;

  static element_container& getElementContainer(domain_type_a& ca, domain_type_b& cb) { return ca; }
  static search_container&  getSearchContainer (domain_type_a& ca, domain_type_b& cb) { return cb; }
};
template <class container_a, class container_b,
          class const_tag
		  >
struct intersection_traits<container_a,container_b,const_tag,
                           InternalFindElement,NoInternalFindElement> {
  typedef view_traits<container_a,const_tag>::container_type domain_type_a;
  typedef view_traits<container_b,const_tag>::container_type domain_type_b;
  typedef domain_type_b element_container;
  typedef domain_type_a search_container;
  typedef InternalFindElement FindType;
  typedef typename std::iterator_traits<typename container_b::iterator>::iterator_category
	  iterator_category;
  
  static element_container& getElementContainer(domain_type_a& ca, domain_type_b& cb) { return cb; }
  static search_container&  getSearchContainer (domain_type_a& ca, domain_type_b& cb) { return ca; } 
};

template <
  class container_a,
  class container_b, 
  class const_tag = const_view_tag,
  template <class C, class tag> class proxy_template_a = view_ref,
  template <class C, class tag> class proxy_template_b = view_ref,
  class FindType_a = ContainerTrait<container_a>::FindType,
  class FindType_b = ContainerTrait<container_b>::FindType,
  class iterator_tag = typename intersection_traits<container_a,container_b,const_tag,
                                                    FindType_a,FindType_b>::iterator_category >
class intersection_view;

END_VIEW_NAMESPACE

#endif // _INTERSECTION_VIEW_FWD_HPP
