/* -*- c++ -*-
 * Copyright (c) 1999  
 * Gary Powell, Martin Weiser 
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 *
 */
#if !defined( _SET_INTERSECTION_VIEW_HPP)
#define _SET_INTERSECTION_VIEW_HPP	1

#if !defined(_SET_INTERSECTION_VIEW_FWD_HPP)
#include "set_intersection_view_fwd.hpp"	// validate that the forward declaration matches the actual one.
#endif
#if !defined(_SORTED_EQUAL_RANGE_BASE_HPP)
#include "sorted_equal_range_base.hpp"
#endif

BEGIN_VIEW_NAMESPACE

// Given a pair of two iterators pointing to ranges of equal values, return the intersection, i.e. a
// subrange of the first range with no more elements that either of the two ranges.
// This is semantically equivalent to the STL set_intersection guarantees.
template <class iter_a_t, class iter_b_t, class const_tag>
class TransformIntersection
  : public std::unary_function<std::pair<iter_a_t,iter_b_t>,iterator_traits<iter_a_t>::value_type> {
  typedef iterator_traits<iter_a_t>::value_type range_a;
  typedef view_traits<range_a,const_tag>::iterator range_a_iterator;
  typedef iterator_traits<iter_a_t>::difference_type difference_type_a;
  typedef iterator_traits<iter_b_t>::difference_type difference_type_b;
public:
  // This is intentionally call by value. Otherwise we could not return non const ranges
  // if desired.
  range_a operator()(std::pair<iter_a_t,iter_b_t> p) const {
    difference_type_a const dist_a = (*(p.first)).size();
    difference_type_b const dist_b = (*(p.second)).size();
    if (dist_a <= dist_b) {
      return *p.first;
    } else {
      range_a_iterator stop = p.first->begin();
      std::advance(stop,dist_b);
      return range_a(p.first->begin(),stop);
    }
  }
};

//
// A sorted_intersection_equal_range_view.
//
// A view with two sorted containers, where an iterator will
// return the next in order item which is only in both of the containers.
//
//
template <class container_a, class container_b,
          class equal_fn_a,
          class equal_fn_b,
	  class comparator,
	  class const_tag,
	  class iterator_tag ,
	  template<class container, class const_tag> class proxy_template_a,
	  template<class container, class const_tag> class proxy_template_b>
class set_intersection_view
: public sorted_equal_range_base_namespace<container_a,container_b,
				             comparator,
                                             equal_fn_a,equal_fn_b,
                                             const_tag,
											 iterator_tag,
                                             proxy_template_a,proxy_template_b,
				             TransformIntersection>::view
{
public:
  typedef typename sorted_equal_range_base_namespace<container_a,container_b,
					    comparator,equal_fn_a,equal_fn_b,const_tag,
						iterator_tag,
					    proxy_template_a,proxy_template_b,
					    TransformIntersection>::view inherited;
  typedef ctor_arg<container_a,const_tag,proxy_template_a>::type ctor_arg_type_a;
  typedef ctor_arg<container_b,const_tag,proxy_template_b>::type ctor_arg_type_b;

  explicit set_intersection_view() {}

  set_intersection_view(ctor_arg_type_a& a_cont, ctor_arg_type_b& b_cont,
                        equal_fn_a const & eq_a = equal_fn_a(),
                        equal_fn_b const & eq_b = equal_fn_b(),
                        comparator const &comp = comparator())
   : inherited::view(a_cont, b_cont, eq_a, eq_b, comp) 
  {}
};

END_VIEW_NAMESPACE
#endif	// _SET_INTERSECTION_VIEW_H
// $Id: set_intersection_view.hpp,v 1.2 2000/04/28 08:20:18 bzfweise Exp $
