/* -*- c++ -*-
 * Copyright (c) 1999  
 * Gary Powell, Martin Weiser
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 *
 */
#if !defined(_STL_TRAITS_HPP)
#define _STL_TRAITS_HPP	1

#if !defined(_STL_TRAITS_FWD_HPP)
#include "stl_traits_fwd.hpp"	// validate that the forward declaration matches the actual one.
#endif

#if !defined USE_FORWARD_DECLARATIONS_OF_STD_CONTAINERS
#include <set>
#include <map>
#endif

#if defined(USING_SGI_STL) && !defined(USE_FORWARD_DECLARATIONS_OF_SGI_NONSTD_CONTAINERS)
#include <hash_map>
#include <hash_set>
#endif


// Forward declarations of the standard containers.
#if defined(OPTIMIZE_COMPILE_TIME)
namespace std {

#if defined(USE_FORWARD_DECLARATIONS_OF_STD_CONTAINERS)
template <class _K, class _Pr, class _A>
class set;

template <class _K, class _Pr, class _A>
class multiset;

template<class _K, class _Ty, class _Pr, class _A>
class map;

template<class _K, class _Ty, class _Pr, class _A>
class multimap;
#endif

#if defined(USING_SGI_STL) && defined(USE_FORWARD_DECLARATIONS_OF_SGI_NONSTD_CONTAINERS)
// Not std but in the SGI version of the STL, and very useful.
template <class _Value, class _HashFcn, class _EqualKey, class _Alloc>
class hash_set;

template <class _Value, class _HashFcn, class _EqualKey, class _Alloc>
class hash_multiset;

template <class _Key, class _Tp, class _HashFcn, class _EqualKey,
          class _Alloc>
class hash_map;

template <class _Key, class _Tp, class _HashFcn, class _EqualKey,
          class _Alloc>
class hash_multimap;
#endif

}
#endif // OPTIMIZE_COMPILE_TIME
BEGIN_VIEW_NAMESPACE
//
// Traits about whether the container has an internal
// find function.
//
class InternalFindElement {};
class NoInternalFindElement {};
class PairInternalEqualRange: public NoInternalFindElement {};
class KeyDataPair {};
class NotKeyDataPair {};


// default container trait.
template<class container>
class ContainerTrait
{
	typedef NoInternalFindElement	FindType;
	typedef NotKeyDataPair          ValuePairType;
};

template<class _K, class _Pr, class _A> 
class ContainerTrait<std::set<_K, _Pr, _A> >
{
	typedef InternalFindElement FindType;
	typedef NotKeyDataPair      ValuePairType;
};

template<class _K, class _Pr, class _A>
class ContainerTrait<std::multiset<_K, _Pr, _A> >
{
	typedef InternalFindElement FindType;
	typedef NotKeyDataPair      ValuePairType;
};

#if defined(USING_SGI_STL)
template <class _Value, class _HashFcn, class _EqualKey, class _Alloc>
class ContainerTrait<SGI_STL_NAMESPACE::hash_set<_Value, _HashFcn, _EqualKey, _Alloc> >
{
	typedef InternalFindElement FindType;
	typedef NotKeyDataPair      ValuePairType;
};

template <class _Value, class _HashFcn, class _EqualKey, class _Alloc>
class ContainerTrait<SGI_STL_NAMESPACE::hash_multiset<_Value, _HashFcn, _EqualKey, _Alloc> >
{
	typedef InternalFindElement FindType;
	typedef NotKeyDataPair      ValuePairType;
};

#endif // USING_SGI_STL
//
// Note: While map, multimap, hash_map, and hash_multimap have an
// internal find function, it finds the "key" not the data element.
// If you want a match of "{1,A}" == "{1,B}" && "1" == "{1,A}" then
// the internal find is the one for you.
template<class _K, class _Ty, class _Pr, class _A>
struct ContainerTrait<std::map<_K, _Ty, _Pr, _A> >
{
  typedef std::map<_K, _Ty, _Pr, _A> container;
  typedef PairInternalEqualRange FindType;
  typedef KeyDataPair            ValuePairType;

  static typename container::key_type const&
  key(typename container::value_type const& value) {
    return value.first;
  }
  
};

template<class _K, class _Ty, class _Pr, class _A>
struct ContainerTrait<std::multimap<_K, _Ty, _Pr, _A> >
{
  typedef std::multimap<_K, _Ty, _Pr, _A> container;
  typedef PairInternalEqualRange FindType;
  typedef KeyDataPair            ValuePairType;

  static typename container::key_type const&
  key(typename container::value_type const& value) {
    return value.first;
  }
};

#if defined(USING_SGI_STL)
template <class _Key, class _Tp, class _HashFcn, class _EqualKey,
          class _Alloc>
struct ContainerTrait<SGI_STL_NAMESPACE::hash_map<_Key, _Tp, _HashFcn, _EqualKey, _Alloc> >
{
  typedef SGI_STL_NAMESPACE::hash_map<_Key, _Tp, _HashFcn, _EqualKey, _Alloc> container;
  typedef PairInternalEqualRange FindType;
  typedef KeyDataPair            ValuePairType;

  static typename container::key_type const&
  key(typename container::value_type const& value) {
    return value.first;
  }
  
};

template <class _Key, class _Tp, class _HashFcn, class _EqualKey,
          class _Alloc>
struct ContainerTrait<std::hash_multimap<_Key, _Tp, _HashFcn, _EqualKey, _Alloc> >
{
  typedef SGI_STL_NAMESPACE::hash_multimap<_Key, _Tp, _HashFcn, _EqualKey, _Alloc> container;
  typedef PairInternalEqualRange FindType;
  typedef KeyDataPair            ValuePairType;

  static typename container::key_type const&
  key(typename container::value_type const& value) {
    return value.first;
  }
};
#endif // USING_SGI_STL

END_VIEW_NAMESPACE

#endif // _STL_TRAITS_H
