/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */
#if !defined (_TWO_CONTAINERS_TRAITS_HPP)
#define _TWO_CONTAINERS_TRAITS_HPP	1

#if !defined(_VIEW_BASE_TRAITS_HPP)
#include "view_base_traits.hpp"
#endif

#if !defined (_TWO_CONTAINERS_BASE_FWD_HPP)
#include "two_containers_base_fwd.hpp"
#endif

BEGIN_VIEW_NAMESPACE

//
// Traits mapping a pair of iterator categories to the smallest xxx_range_own
// range that does not artificailly limit the possibilities of the
// iterator it's based on.
//

template <class iterator_category, class container_a, class container_b, class const_tag>
struct internal_two_range_traits { 
  typedef view_traits<container_a,const_tag>::iterator       domain_a_iterator;
  typedef view_traits<container_a,const_tag>::const_iterator domain_a_const_iterator;
  typedef view_traits<container_b,const_tag>::iterator       domain_b_iterator;
  typedef view_traits<container_b,const_tag>::const_iterator domain_b_const_iterator;

  typedef unidirectional_base_containers_range_own<domain_a_iterator,
													domain_b_iterator>      type; 
  typedef unidirectional_base_containers_range_own<domain_a_const_iterator,
													domain_b_const_iterator> const_type; 
};

template <class container_a, class container_b, class const_tag>
struct internal_two_range_traits<std::bidirectional_iterator_tag, container_a, container_b, const_tag> {
  typedef view_traits<container_a,const_tag>::iterator       domain_a_iterator;
  typedef view_traits<container_a,const_tag>::const_iterator domain_a_const_iterator;
  typedef view_traits<container_b,const_tag>::iterator       domain_b_iterator;
  typedef view_traits<container_b,const_tag>::const_iterator domain_b_const_iterator;

  typedef two_containers_range_own<domain_a_iterator,
									domain_b_iterator>      type; 
  typedef two_containers_range_own<domain_a_const_iterator,
									domain_b_const_iterator> const_type; 
};

template <class container_a, class container_b, class const_tag>
struct internal_two_range_traits<std::random_access_iterator_tag, container_a, container_b, const_tag> {
  typedef view_traits<container_a,const_tag>::iterator       domain_a_iterator;
  typedef view_traits<container_a,const_tag>::const_iterator domain_a_const_iterator;
  typedef view_traits<container_b,const_tag>::iterator       domain_b_iterator;
  typedef view_traits<container_b,const_tag>::const_iterator domain_b_const_iterator;

// This specialization may be wrong, I'm not sure but I think we should be using the
// two_containers_base_ref GWP.
  typedef two_containers_range_own<domain_a_iterator,
									domain_b_iterator>      type; 
  typedef two_containers_range_own<domain_a_const_iterator,
									domain_b_const_iterator> const_type; 
};

template <class container_a, class container_b, class const_tag>
struct two_containers_range_traits {
  typedef combine_iterator_tags<typename container_a::iterator,
								typename container_b::iterator>::type category;
 
  typedef internal_two_range_traits<category,container_a,container_b,const_tag>::type type; 
};

END_VIEW_NAMESPACE


#endif	// _TWO_CONTAINERS_TRAITS_HPP
