/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

#ifndef _VIEW_CONFIG_HPP
#define _VIEW_CONFIG_HPP

#if defined(__GNUC__)
#define USING_SGI_STL
// GCC Has namespaces but 2.95.2 is broken.
// (Internal compiler errors.)
#if __GNUC__ <= 2 && __GNUC_MINOR__ <= 95
#define FRIEND_TEMPLATE_DECLARATION_COMPILES
#else
#define USE_VIEW_NAMESPACE
#define FRIEND_TEMPLATE_DECLARATION_COMPILES
#endif
#endif // if __GNUC__

// For gcc 2.95.2 you can enable USE_VIEW_NAMESPACE, but you
// disable FRIEND_TEMPLATE_DECLARATION_COMPILES.
// It will then use namespaces, but inorder to do a
// const comparison to non-const, all iterator member
// variables become public. (The namespace/friend template
// compiler code is broken badly, and will crash the compiler.)

//#define USE_VIEW_NAMESPACE
//#undef FRIEND_TEMPLATE_DECLARATION_COMPILES


#if defined(USE_VIEW_NAMESPACE)
#define VIEW_NAMESPACE_NAME      boost
#define VIEW_NAMESPACE ::boost
#define BEGIN_VIEW_NAMESPACE namespace boost {
#define END_VIEW_NAMESPACE }
#define USING_VIEW_NAMESPACE using namespace boost;
#else
#define VIEW_NAMESPACE
#define BEGIN_VIEW_NAMESPACE
#define END_VIEW_NAMESPACE
#define USING_VIEW_NAMESPACE
#endif

// Turn OPTIMIZE_COMPILE_TIME on IF
// 1) You want to speed up your compile time by reducing unnecessary parsing.
// 2) Recognize that by using forward declarations of the std
//    containers/algorithms is NOT supported by the ISO C++ standard.
// 3) Your compiler does the right thing with reguard to recognizing these
//    forward declarations.
// 4) And you don't mind tracking down which std include files you need in
//    addition to the view files. (To define a view it shouldn't be necessary
//    for inclusion in your include files, to use one may require it, as in the .cpp file.)
// 5) Additionally if you use operator == it may need lexigraphical_compare, which would
//    require you to include <functional>
//#define OPTIMIZE_COMPILE_TIME

#if defined(OPTIMIZE_COMPILE_TIME)
#define USE_FORWARD_DECLARATIONS_OF_STD_CONTAINERS

// Are you using the SGI STL?
#if defined(USING_SGI_STL)
// Are you using hash_*?
#define USE_FORWARD_DECLARATIONS_OF_SGI_NONSTD_CONTAINERS
#endif // USING_SGI_STL

#define USE_FORWARD_DECLARATIONS_OF_STD_ALGORITHM
#define USE_FORWARD_DECLARATIONS_OF_STD_ITERATOR
#define USE_FORWARD_DECLARATIONS_OF_STD_FUNCTIONAL

#endif // OPTIMIZE_COMPILE_TIME

// Are you using the SGI STL?
#if defined(USING_SGI_STL)
// used by hash_* containers.
#define SGI_STL_NAMESPACE    std
#endif // USING_SGI_STL

// Turn this flag on if you want std::swap(view&,view&)
// to call viewNamespace::swap(view&,view&);
// This is not standard C++ as of C++99 ISO.
// Otherwise do the following:
// using std::swap;
// swap(view&,view&);
// which if a viewNamespace::swap exists will call it, otherwise
// std::swap will be called. (This is the recommended way to code a swap.)
// WARNING: use of std::swap may not give you the desired swap.
//          and will silently compile!
//
// Also some compilers which have swap() in the global namespace instead
// of std:: will have ambigious calls to swap(). Use SWAP_NAMESPACE::swap
// to resolve these calls.

//#define USE_NON_STANDARD_SWAP


#if defined(USE_NON_STANDARD_SWAP)
#define MEMBER_SWAP_NAMESPACE ::std
#define BEGIN_SWAP_NAMESPACE namespace std {
#define END_SWAP_NAMESPACE }
#define SWAP_NAMESPACE ::std
#else
#define MEMBER_SWAP_NAMESPACE VIEW_NAMESPACE
#define BEGIN_SWAP_NAMESPACE BEGIN_VIEW_NAMESPACE
#define END_SWAP_NAMESPACE END_VIEW_NAMESPACE
#define SWAP_NAMESPACE VIEW_NAMESPACE
#endif

#endif
