/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

#include "rotate_view.hpp"

#include <cstdlib>
#include <vector>
#include <iostream>
#include <numeric>

#include "dump.h"
#include "iterator_test.h"

using std::cout;
using std::cin;
using std::endl;
using std::vector;
using std::iota;
using std::advance;

USING_VIEW_NAMESPACE

int main()
{
	vector<int> x(14);

	iota (x.begin(), x.end(), 0);

	// bidirectional version.
	typedef rotate_view<vector<int>::iterator, vector<int>::const_iterator, 
			const_view_tag> rotate_vv_view;

	rotate_vv_view rvv(x.begin() + 1, x.end() - 1, 
					x.begin() + (x.size()/2) );

    cout << "x set" << endl;
    dump(cout, x);

	cout << "Rotate Test, x.begin + 1 -> x.end() - 1." << endl;
    dump(cout, rvv);

	cout << "Reverse Rotate Test, x.begin + 1 -> x.end() - 1." << endl;
    rdump(cout, rvv);

	cout << "Advance 4 from the beginning." << endl;
	rotate_vv_view::iterator r_iter(rvv.begin() );
	advance(r_iter, 4);
	cout << *r_iter << endl;

	cout << "Reverse 4, back to the beginning." << endl;
	advance(r_iter, -4);
	cout << *r_iter << endl;

	cout << "Advance 9 from the beginning." << endl;
	r_iter = rvv.begin();
	advance(r_iter, 9);
	cout << *r_iter << endl;

	cout << "Reverse 9, back to the beginning." << endl;
	advance(r_iter, -9);
	cout << *r_iter << endl;

	forward_test(rvv);
  const_forward_test(rvv);
  reverse_test(rvv);
  const_reverse_test(rvv);

  // random access version.
  typedef rotate_view<vector<int>::iterator, vector<int>::const_iterator, 
			const_view_tag> rrotate_vv_view;

  rrotate_vv_view rrvv(x.begin() + 1, x.end() - 1, 
					x.begin() + (x.size()/2) );

  random_access_test(rrvv);
  const_random_access_test(rrvv);



  cout << "type q <cr> to exit app. ";
  char c;
  do {
    cin >> c;
    cout << c;
  } while (c != 'q');
    
  cout << endl;
  return EXIT_SUCCESS;
}

